@@ -2,7 +2,7 @@
 **
 ** Copyright (C) 1994 Swedish University Network (SUNET)
 ** Modified by Rami Lehti (C) 1999
-** $Header: /cvs-root-aide/aide/include/base64.h,v 1.5 2000/04/20 12:44:49 rammer Exp $
+** $Header: /cvs-root-aide/aide2/include/base64.h,v 1.2 2002/03/18 15:18:45 rammer Exp $
 **
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -45,5 +45,7 @@ char* encode_base64(byte* src,size_t ssize);
 
 byte* decode_base64(char* src,size_t ssize);
 
+/* Returns decoded length */
+size_t length_base64(char* src,size_t ssize);
 
 #endif /* _BASE64_H_INCLUDED */
