@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/commandconf.h,v 1.10 2000/04/20 12:44:49 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/commandconf.h,v 1.11 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,9 @@ extern long conf_lineno;
 
 int commandconf(const char mode,const char* line);
 
+int conf_input_wrapper(char* buf, int max_size, FILE* in);
+int db_input_wrapper(char* buf, int max_size, int db);
+
 list* append_rxlist(char*,int,list*);
 
 void do_define(char*,char*);
@@ -45,9 +48,7 @@ void putbackvariable(char*);
 
 int handle_endif(int doit,int allow_else);
 
-void do_dbindef(char*);
-
-void do_dboutdef(char*);
+void do_dbdef(int, char*);
 
 void do_verbdef(char*);
 
@@ -57,4 +58,34 @@ void do_repurldef(char*);
 
 int check_db_order(DB_FIELD*,int, DB_FIELD);
 
+void* get_db_key();
+void* get_conf_key();
+size_t get_db_key_len();
+size_t get_conf_key_len();
+
+
+
+extern const char* aide_key_1;
+extern const char* aide_key_2;
+extern const char* aide_key_3;
+extern const char* aide_key_4;
+extern const char* aide_key_5;
+extern const char* aide_key_6;
+extern const char* aide_key_7;
+extern const char* aide_key_8;
+extern const char* aide_key_9;
+extern const char* aide_key_0;
+
+extern const char* db_key_1;
+extern const char* db_key_2;
+extern const char* db_key_3;
+extern const char* db_key_4;
+extern const char* db_key_5;
+extern const char* db_key_6;
+extern const char* db_key_7;
+extern const char* db_key_8;
+extern const char* db_key_9;
+extern const char* db_key_0;
+
+
 #endif
