@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/compare_db.h,v 1.5 2000/04/20 12:44:50 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/compare_db.h,v 1.2 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,32 @@
 #define _COMPARE_DB_H_INCLUDED
 #include "list.h"
 #include "db_config.h"
+#include "seltree.h"
+
+/*
+ * compare_db()
+ * Generate report based on list of files and db from disk
+ */
 
 void compare_db(list*,db_config*);
 
+/*
+ * report_tree()
+ * Generate report based on the tree left by populate_tree()
+ * stage= which stage to do check=0 added=1, removed=2
+ * changed=3 , detailed=4
+ * stat contains 5 ints, 0: are we the first to be called
+ * 1: total number of files 2: # added files, 
+ * 3: # removed, 4: # changed
+ */
+long report_tree(seltree* tree,int stage, long* stat);
+
+void init_rxlst(list* rxlst);
+
+/* 
+ * compare_dbline()
+ * Return RETOK if same RETFAIL if not
+ */
+int compare_dbline(db_line* old,db_line* new,int ignorelist);
+
 #endif
