@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/conf_lex.h,v 1.4 2000/04/20 12:44:50 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/conf_lex.h,v 1.3 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,4 +27,8 @@ extern int conferror(const char*);
 
 extern int conflex();
 
+extern int confparse();
+
+extern void* conf_scan_string(char*);
+
 #endif
