@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/db.h,v 1.8 2000/07/28 12:32:20 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/db.h,v 1.3 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,9 +24,9 @@
 #include <stdio.h>
 #include "db_config.h"
 
-int db_init(db_config*,int);
+int db_init(int);
 
-db_line* db_readline(db_config*);
+db_line* db_readline(int);
 
 int db_writespec(db_config*);
 
@@ -34,7 +34,22 @@ int db_writeline(db_line*,db_config*);
 
 int db_close(db_config*);
 
+void free_db_line(db_line* dl);
+
 extern const char* db_names[];
 extern const int db_value[];
 
+#define DB_OLD            (1<<0)
+#define DB_WRITE          (1<<1)
+#define DB_NEW            (1<<2)
+#define NODE_ADDED        (1<<4)
+#define NODE_REMOVED      (1<<5)
+#define NODE_CHANGED      (1<<6)
+#define NODE_ADD_CHILDREN (1<<7)
+#define DB_DISK           (1<<8)
+
+#define NODE_TRAVERSE     (1<<9)
+#define NODE_CHECKED      (1<<10)
+
+
 #endif
