@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/db_file.h,v 1.8 2002/02/10 19:10:32 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/db_file.h,v 1.2 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,12 +23,20 @@
 
 #include "db.h"
 
-extern long db_lineno;
+extern void db_buff(int, FILE*);
+extern int db_scan(); /* Rumaa.... */
+extern char* dbtext; /* Todella rumaa... */
+extern long* db_lineno;
+extern long db_in_lineno;
+extern long db_new_lineno;
 
-char** db_readline_file(db_config*);
+
+char** db_readline_file(int);
 int db_writespec_file(db_config*);
 int db_writeline_file(db_line* line,db_config* conf);
 int db_close_file(db_config* conf);
-void handle_gzipped_input(int out);
+#ifdef WITH_ZLIB
+void handle_gzipped_input(int out,gzFile*);
+#endif
 
 #endif
