@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/db_sql.h,v 1.8 2002/02/10 17:31:29 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/db_sql.h,v 1.2 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,6 +50,6 @@ const static char* db_sql_types[] = {
 };
 
 int db_close_sql(void*);
-db_line* db_readline_sql(db_config* conf);
+db_line* db_readline_sql(int db);
 
 #endif
