@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/gen_list.h,v 1.11 2002/02/10 17:31:29 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/gen_list.h,v 1.6 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 
 #ifndef _GEN_LIST_H_INCLUDED
 #define _GEN_LIST_H_INCLUDED
+#include "seltree.h"
 #include "list.h"
 #include "gnu_regex.h"
 
@@ -40,6 +41,20 @@ typedef struct rx_rule {
 
 list* gen_list(list* prxlist,list* nrxlist,list* erxlist);
 
+/* 
+ * gen_tree()
+ * Generates the file tree
+ * from rx_rule's
+ */
+seltree* gen_tree(list* prxlist,list* nrxlist,list* erxlist);
+
+/* 
+ * populate_tree()
+ * Populate tree with data from disk and db 
+ * Also do comparing while adding to the tree
+ */
+void populate_tree(seltree* tree);
+
 /*
  * strrxtok()
  * return a pointer to a copy of the non-regexp path part of the argument
@@ -49,4 +64,10 @@ char* strrxtok(char*);
 
 int check_list_for_match(list*,char*,int*);
 
+int check_rxtree(char* filename,seltree* tree, int* attr);
+
+db_line* get_file_attrs(char* filename,int attr);
+
+seltree* get_seltree_node(seltree* tree,char* path);
+
 #endif /*_GEN_LIST_H_INCLUDED*/
