@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/report.h,v 1.2 2000/04/20 12:44:52 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/report.h,v 1.3 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,7 +37,7 @@
 
 void error(int errorlevel, char* error_msg,...);
 
-int error_init(url_t*);
+int error_init(url_t*,int);
 
 void write_error_stderr(int errorlevel, char*error_msg,va_list ap);
 
