@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/seltree.h,v 1.9 2002/02/10 17:31:29 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/include/seltree.h,v 1.3 2002/05/29 08:04:26 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,15 +20,21 @@
 
 #ifndef _SELTREE_H_INCLUDED
 #define _SELTREE_H_INCLUDED
+struct seltree;
+#include "db_config.h"
 #include "list.h"
 
 
 /* seltree structure
  * lists have regex_t* in them
- * checked is whether or not the node has been checked yet
+ * checked is whether or not the node has been checked yet and status
+ * when added  
  * path is the path of the node
  * parent is the parent, NULL if root
  * childs is list of seltree*:s
+ * new_data is this nodes new attributes (read from disk or db in --compare)
+ * old_data is this nodes old attributes (read from db)
+ * attr attributes to add for this node and possibly for its children
  */
 
 typedef struct seltree {
@@ -44,6 +50,11 @@ typedef struct seltree {
   long      conf_lineno;
   char*     rx;    
 
+  int attr;
+
+  struct db_line* new_data;
+  struct db_line* old_data;
+
 } seltree;
 
 #endif /* _SELTREE_H_INCLUDED*/
