@@ -2,7 +2,7 @@
 **
 ** Copyright (C) 1994 Swedish University Network (SUNET)
 ** Modified by Rami Lehti (C) 1999
-** $Header: /cvs-root-aide/aide/src/base64.c,v 1.12 2000/05/29 08:38:55 rammer Exp $
+** $Header: /cvs-root-aide/aide2/src/base64.c,v 1.3 2002/03/18 15:18:45 rammer Exp $
 **
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 **
 */
 
-
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "config.h"
@@ -255,3 +255,73 @@ byte* decode_base64(char* src,size_t ssize)
   return retbuf;
 }
 
+size_t length_base64(char* src,size_t ssize)
+{
+  char* inb;
+  int i;
+  int l;
+  int left;
+  size_t pos;
+  unsigned long triple;
+
+  error(210, "decode base64\n");
+  /* Exit on empty input */
+  if (!ssize||src==NULL)
+    return NULL;
+
+
+
+  /* Initialize working pointers */
+  inb = src;
+
+  l = 0;
+  triple = 0;
+  pos=0;
+  left = ssize;
+  /*
+   * Process entire inbuf.
+   */
+  while (left != 0)
+    {
+      left--;
+      i = fromb64[(unsigned char)*inb];
+      switch(i)
+	{
+	case FAIL:
+	  error(5, "length_base64: Illegal character: %c\n", *inb);
+	  error(230, "length_base64: Illegal line:\n%s\n", src);
+	  return NULL;
+	  break;
+	case SKIP:
+	  break;
+	default:
+	  triple = triple<<6 | (0x3f & i);
+	  l++;
+	  break;
+	}
+      if (l == 4 || left == 0)
+	{
+	  switch(l)
+	    {
+	    case 2:
+	      triple = triple>>4;
+	      break;
+	    case 3:
+	      triple = triple>>2;
+	      break;
+	    default:
+	      break;
+	    }
+	  for (l  -= 2; l >= 0; l--)
+	    {
+	      pos++;
+	    }
+	  triple = 0;
+	  l = 0;
+	}
+      inb++;
+    }
+  
+  return pos;
+}
+
