@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/compare_db.c,v 1.44 2002/02/10 17:31:30 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/src/compare_db.c,v 1.9 2002/05/30 09:42:46 pablo Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,11 +38,7 @@
 #include "locale-aide.h"
 /*for locale support*/
 
-#ifdef WITH_MHASH
-#include <mhash.h>
-#endif
-
-#include <gcrypt.h>
+#include "md.h"
 
 /*************/
 /* contruction area for report lines */
@@ -119,6 +115,9 @@ int compare_acl(acl_type* a1,acl_type* a2) {
 
 int compare_md_entries(byte* e1,byte* e2,int len)
 {
+
+  error(255,"Debug, compare_md_entreis %p %p\n",e1,e2);
+
   if(e1!=NULL && e2!=NULL){
     if(strncmp(e1,e2,len)!=0){
       return RETFAIL;
@@ -136,150 +135,109 @@ int compare_md_entries(byte* e1,byte* e2,int len)
   return RETFAIL;
 }
 
-int compare_lines(db_line* l1,db_line* l2,int ignorelist)
+
+/*
+  We assume
+  - no null parameters
+  - same filename
+  - something else?
+  - ignorelist kertoo mit ei saa vertailla
+*/
+
+int compare_dbline(db_line* l1,db_line* l2,int ignorelist)
 {
+
+#define easy_compare(a,b) \
+  if (!(a&ignorelist)) {\
+    if(l1->b!=l2->b){\
+      ret|=a;\
+    }\
+  }
+
+#define easy_md_compare(a,b,c) \
+  if (!(a&ignorelist)) {  \
+    if(compare_md_entries(l1->b,l2->b,\
+			  c)==RETFAIL){ \
+      ret|=a; \
+    } \
+  }
+  
+  
+  int ret=0;
+  
   if (!(DB_LINKNAME&ignorelist)) {
     if(l1->linkname==NULL){
       if(l2->linkname!=NULL){
-	return RETFAIL;
+	ret|=DB_LINKNAME;
+	//return RETFAIL;
       }
     }else if(l2->linkname==NULL){
-      return RETFAIL;
+	ret|=DB_LINKNAME;
+	//return RETFAIL;
     }else if(strcmp(l1->linkname,l2->linkname)!=0){
-      return RETFAIL;
+	ret|=DB_LINKNAME;
+	//return RETFAIL;
     }
   }
     
   if (!(DB_SIZEG&ignorelist)) {
     if ( (DB_SIZEG&l2->attr) && !(DB_SIZE&l2->attr) ){
       if(l1->size>l2->size){
-	return RETFAIL;
+	ret|=DB_SIZEG;
+	//return RETFAIL;
       }
     } else {
       if(l1->size!=l2->size){
-	return RETFAIL;
+	ret|=DB_SIZEG;
+	//return RETFAIL;
       }
     }
   }
-  if (!(DB_BCOUNT&ignorelist)) {
-    if(l1->bcount!=l2->bcount){
-      return RETFAIL;
-    }
-  }
+  
+  easy_compare(DB_BCOUNT,bcount);
+  
   if (!(DB_PERM&ignorelist)) {
     if(l1->perm!=l2->perm){
-      return RETFAIL;
+      ret|=DB_PERM;
+      //return RETFAIL;
     }
   } else {
     error(0,"Ignoring permissions\n");
   }
-  if (!(DB_UID&ignorelist)) {
-    if(l1->uid!=l2->uid){
-      return RETFAIL;
-    }
-  }
-  if (!(DB_GID&ignorelist)) {
-    if(l1->gid!=l2->gid){
-      return RETFAIL;
-    }
-  }
+  
+  easy_compare(DB_UID,uid);
+  easy_compare(DB_GID,gid);
+  easy_compare(DB_ATIME,atime);
+  easy_compare(DB_MTIME,mtime);
+  easy_compare(DB_CTIME,ctime);
 
-  if (!(DB_ATIME&ignorelist)) {
-    if(l1->atime!=l2->atime){
-      return RETFAIL;
-    }
-  }
-  
-  if (!(DB_MTIME&ignorelist)) {
-    if(l1->mtime!=l2->mtime){
-      return RETFAIL;
-    }
-  }
-  if (!(DB_CTIME&ignorelist)) {
-    if(l1->ctime!=l2->ctime){
-      return RETFAIL;
-    }
-  }
-  
-  if (!(DB_INODE&ignorelist)) {
-    if(l1->inode!=l2->inode){
-      return RETFAIL;
-    }
-  }
-  
-  if (!(DB_LNKCOUNT&ignorelist)) {
-    if(l1->nlink!=l2->nlink){
-      return RETFAIL;
-    }
-  }
-  
-  if (!(DB_MD5&ignorelist)) {  
-    if(compare_md_entries(l1->md5,l2->md5,
-			  gcry_md_get_algo_dlen(GCRY_MD_MD5))==RETFAIL){
-      return RETFAIL;
-    }
-  }
 
-  if (!(DB_SHA1&ignorelist)) {
-    if(compare_md_entries(l1->sha1,l2->sha1,
-			  gcry_md_get_algo_dlen(GCRY_MD_SHA1))==RETFAIL){
-      return RETFAIL;
-    }
-  }
+  easy_compare(DB_INODE,inode);
+  easy_compare(DB_LNKCOUNT,nlink);
+
+  easy_md_compare(DB_MD5,md5,HASH_MD5_LEN);
   
-  if (!(DB_RMD160&ignorelist)) {
-    if(compare_md_entries(l1->rmd160,l2->rmd160,
-			  gcry_md_get_algo_dlen(GCRY_MD_RMD160))==RETFAIL){
-      return RETFAIL;
-    }
-  }
+  error(255,"Debug, %s, %p %p %i %i\n",l1->filename,l1->md5,l2->md5,ret&DB_MD5,ignorelist);
   
-  if (!(DB_TIGER&ignorelist)) {
-    if(compare_md_entries(l1->tiger,l2->tiger,
-			  gcry_md_get_algo_dlen(GCRY_MD_TIGER))==RETFAIL){
-      return RETFAIL;
-    }
-  }
+  easy_md_compare(DB_SHA1,sha1,HASH_SHA1_LEN);
+  easy_md_compare(DB_RMD160,rmd160,HASH_RMD160_LEN);
+  easy_md_compare(DB_TIGER,tiger,HASH_TIGER_LEN);
   
 #ifdef WITH_MHASH
-  if (!(DB_CRC32&ignorelist)) {
-    if(compare_md_entries(l1->crc32,l2->crc32,
-			  mhash_get_block_size(MHASH_CRC32))==RETFAIL){
-      return RETFAIL;
-    }
-  }
-  
-  if (!(DB_HAVAL&ignorelist)) {
-    if(compare_md_entries(l1->haval,l2->haval,
-			  mhash_get_block_size(MHASH_HAVAL256))==RETFAIL){
-      return RETFAIL;
-    }
-  }
-    
-  if (!(DB_GOST&ignorelist)) {
-    if(compare_md_entries(l1->gost,l2->gost,
-			  mhash_get_block_size(MHASH_GOST))==RETFAIL){
-      return RETFAIL;
-    }
-  }
-  
-  if (!(DB_CRC32B&ignorelist)) {
-    if(compare_md_entries(l1->crc32b,l2->crc32b,
-			  mhash_get_block_size(MHASH_CRC32B))==RETFAIL){
-      return RETFAIL;
-    }
-  }
+  easy_md_compare(DB_CRC32,crc32,HASH_CRC32_LEN);
+  easy_md_compare(DB_HAVAL,haval,HASH_HAVAL256_LEN);
+  easy_md_compare(DB_GOST,gost,HASH_GOST_LEN);
+  easy_md_compare(DB_CRC32B,crc32b,HASH_CRC32B_LEN);
 #endif
 
 #ifdef WITH_ACL
   if (!(DB_ACL&ignorelist)) {
-    if(compare_acl(l1->acl,l2->acl)==RETFAIL) {
-      return RETFAIL;
+    if(compare_acl(l1->acl,l2->acl)) {
+      ret|=DB_ACL;
     }
   }
 #endif
-  
-  return RETOK;
+  return ret;
 }
 
 void print_lname_changes(char*old,char*new)
@@ -303,6 +261,8 @@ void print_lname_changes(char*old,char*new)
   }
    if(ok)
      error(5,(char*)entry_format,"Lname",oline,nline);
+
+   return;
 }
 
 #ifdef WITH_ACL
@@ -340,9 +300,9 @@ void print_md_changes(byte*old,byte*new,int len,char* name)
   int ok = 0;
   if(old!=NULL && new!=NULL){
     if(strncmp(old,new,len)!=0){
-    	snprintf(oline,part_len,"%s",encode_base64(old,len));
-	snprintf(nline,part_len,"%s",encode_base64(new,len));
-	ok = 1;
+      snprintf(oline,part_len,"%s",encode_base64(old,len));
+      snprintf(nline,part_len,"%s",encode_base64(new,len));
+      ok = 1;
     }
   } else {
     if(old == NULL && new == NULL){
@@ -363,9 +323,11 @@ void print_md_changes(byte*old,byte*new,int len,char* name)
     }
   }
   if(ok)
-     error(5,(char*)entry_format,name,oline,nline);
-  }
+    error(5,(char*)entry_format,name,oline,nline);
   
+  return;
+}
+
 int is_time_null(struct tm *ot)
 {
     /* 1970-01-01 01:00:00 is year null */
@@ -388,19 +350,19 @@ void print_time_changes(const char* name, time_t old_time, time_t new_time)
   nt = localtime(&(new_time));
   
   if( is_time_null(ot) )
-      snprintf(oline,part_len,"NA");
+    snprintf(oline,part_len,"NA");
   else
-      snprintf(oline,part_len,
-               "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
-           ot->tm_year+1900, ot->tm_mon+1, ot->tm_mday,
-               ot->tm_hour, ot->tm_min, ot->tm_sec);
+    snprintf(oline,part_len,
+	     "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
+	     ot->tm_year+1900, ot->tm_mon+1, ot->tm_mday,
+	     ot->tm_hour, ot->tm_min, ot->tm_sec);
   if( is_time_null(nt) )
-      snprintf(nline,part_len,"NA");
+    snprintf(nline,part_len,"NA");
   else
-      snprintf(nline,part_len,
-               "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
-	nt->tm_year+1900, nt->tm_mon+1, nt->tm_mday,
-	nt->tm_hour, nt->tm_min, nt->tm_sec);
+    snprintf(nline,part_len,
+	     "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
+	     nt->tm_year+1900, nt->tm_mon+1, nt->tm_mday,
+	     nt->tm_hour, nt->tm_min, nt->tm_sec);
   error(5,(char*)entry_format,name,oline,nline); 
 }
 
@@ -441,6 +403,7 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
 {
   char* tmp=NULL;
   char* tmp2=NULL;
+  
 
   if(S_ISDIR(new->perm_o)){
     error(5,"\nDirectory: %s\n",old->filename);
@@ -453,13 +416,13 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
   }
   if (!(DB_SIZE&ignorelist)) {
     if(old->size!=new->size){
-        print_ulong_changes("Size", old->size,new->size);
+      print_ulong_changes("Size", old->size,new->size);
     }
   }
 
   if (!(DB_BCOUNT&ignorelist)) {
     if(old->bcount!=new->bcount){
-          print_int_changes("Bcount", old->bcount,new->bcount);
+      print_int_changes("Bcount", old->bcount,new->bcount);
     }
   }
   if (!(DB_PERM&ignorelist)) {
@@ -476,13 +439,13 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
   
   if (!(DB_UID&ignorelist)) {
     if(old->uid!=new->uid){
-        print_int_changes("Uid", old->uid,new->uid);
+      print_int_changes("Uid", old->uid,new->uid);
     }
   }
-
+  
   if (!(DB_GID&ignorelist)) {
     if(old->gid!=new->gid){
-        print_int_changes("Gid", old->gid,new->gid);
+      print_int_changes("Gid", old->gid,new->gid);
     }
   }
   
@@ -506,61 +469,61 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
 
   if (!(DB_INODE&ignorelist)) {
     if(old->inode!=new->inode){
-        print_int_changes("Inode", old->inode,new->inode);
+      print_int_changes("Inode", old->inode,new->inode);
     }
   }
   if (!(DB_LNKCOUNT&ignorelist)) {
     if(old->nlink!=new->nlink){
-        print_int_changes("Linkcount", old->nlink,new->nlink);
+      print_int_changes("Linkcount", old->nlink,new->nlink);
     }
   }
 
   if (!(DB_MD5&ignorelist)) {  
     print_md_changes(old->md5,new->md5,
-		     gcry_md_get_algo_dlen(GCRY_MD_MD5),
+		     HASH_MD5_LEN,
 		     "MD5");
   }
   
   if (!(DB_SHA1&ignorelist)) {
-    print_md_changes(old->sha1,new->sha1,
-		     gcry_md_get_algo_dlen(GCRY_MD_SHA1),
-		     "SHA1");
+      print_md_changes(old->sha1,new->sha1,
+		       HASH_SHA1_LEN,
+		       "SHA1");
   }
 
   if (!(DB_RMD160&ignorelist)) {
     print_md_changes(old->rmd160,new->rmd160,
-		     gcry_md_get_algo_dlen(GCRY_MD_RMD160),
+		     HASH_RMD160_LEN,
 		     "RMD160");
   }
   
   if (!(DB_TIGER&ignorelist)) {
     print_md_changes(old->tiger,new->tiger,
-		     gcry_md_get_algo_dlen(GCRY_MD_TIGER),
+		     HASH_TIGER_LEN,
 		     "TIGER");
   }
   
 #ifdef WITH_MHASH
   if (!(DB_CRC32&ignorelist)) {
     print_md_changes(old->crc32,new->crc32,
-		     mhash_get_block_size(MHASH_CRC32),
+		     HASH_CRC32_LEN,
 		     "CRC32");
   }
-
+  
   if (!(DB_HAVAL&ignorelist)) {
     print_md_changes(old->haval,new->haval,
-		     mhash_get_block_size(MHASH_HAVAL256),
+		     HASH_HAVAL256_LEN,
 		     "HAVAL");
   }
   
   if (!(DB_GOST&ignorelist)) {
     print_md_changes(old->gost,new->gost,
-		     mhash_get_block_size(MHASH_GOST),
+		     HASH_GOST_LEN,
 		     "GOST");
   }
   
   if (!(DB_CRC32B&ignorelist)) {
     print_md_changes(old->crc32b,new->crc32b,
-		     mhash_get_block_size(MHASH_CRC32B),
+		     HASH_CRC32B_LEN,
 		     "CRC32B");
   }
 #endif                   
@@ -570,43 +533,8 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
     print_acl_changes(old->acl,new->acl);
   }
 #endif
-
-}
-
-void free_db_line(db_line* dl)
-{
-
-#define checked_free(x) if(x) free(x)
-
-  checked_free(dl->md5);
-  checked_free(dl->sha1);
-  checked_free(dl->rmd160);
-  checked_free(dl->tiger);
-  checked_free(dl->filename);
-  checked_free(dl->linkname);
   
-
-#ifdef WITH_MHASH
-  checked_free(dl->crc32);
-  checked_free(dl->crc32b);
-  checked_free(dl->gost);
-  checked_free(dl->haval);
-
-  dl->crc32=NULL;
-  dl->crc32b=NULL;
-  dl->gost=NULL;
-  dl->haval=NULL;
-#endif
-
-
-
-  dl->filename=NULL;
-  dl->linkname=NULL;
-  dl->md5=NULL;
-  dl->sha1=NULL;
-  dl->rmd160=NULL;
-  dl->tiger=NULL;
-
+  return;
 }
 
 void init_rxlst(list* rxlst)
@@ -640,6 +568,7 @@ void init_rxlst(list* rxlst)
       error(0,_("Error in selective regexp:%s"),((rx_rule*)r->data)->rx);
       free(data);
     }else {
+      rxrultmp->conf_lineno=((rx_rule*)r)->conf_lineno;
       free(rxrultmp->rx);
       rxrultmp->rx=data;
       rxrultmp->crx=rxtmp;
@@ -668,6 +597,32 @@ void eat_files_indir(list* flist,char* dirname,long* filcount)
   }
 }
 
+void print_report_header(int nfil,int nadd,int nrem,int nchg)
+{
+  struct tm* st=localtime(&(conf->start_time));
+  if(conf->action&DO_COMPARE)
+    error(0,_("AIDE found differences between database and filesystem!!\n"));
+
+  if(conf->action&DO_DIFF)
+    error(0,_("AIDE found differences between the two databases!!\n"));
+  if(conf->config_version)
+    error(5,_("Config version used: %s\n"),conf->config_version);
+
+  error(5,_("Start timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
+	st->tm_year+1900, st->tm_mon+1, st->tm_mday,
+	st->tm_hour, st->tm_min, st->tm_sec);
+  error(0,_("Summary:\nTotal number of files=%i,added files=%i"
+	    ",removed files=%i,changed files=%i\n\n"),nfil,nadd,nrem,nchg);
+  
+}
+
+void print_report_footer(struct tm* st)
+{
+    error(20,_("\nEnd timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
+	  st->tm_year+1900, st->tm_mon+1, st->tm_mday,
+	  st->tm_hour, st->tm_min, st->tm_sec);
+}
+
 void compare_db(list* new,db_config* conf)
 {
   db_line* old=NULL;
@@ -709,7 +664,7 @@ void compare_db(list* new,db_config* conf)
     ignorelist=0;
   }
 
-  for(old=db_readline(conf);old;old=db_readline(conf)){
+  for(old=db_readline(DB_OLD);old;old=db_readline(DB_OLD)){
     nfil++;
     r=find_line_match(old,l);
     if(r==NULL){
@@ -731,12 +686,12 @@ void compare_db(list* new,db_config* conf)
       int localignorelist=old->attr ^ ((db_line*)r->data)->attr;
       
       if (localignorelist!=0) {
-	error(21,"File %s in databases has different attributes, %i,%i\n",old->filename,old->attr,((db_line*)r->data)->attr);
+	error(5,"File %s in databases has different attributes, %i,%i\n",old->filename,old->attr,((db_line*)r->data)->attr);
       }
       
       localignorelist|=ignorelist;
       
-      if(compare_lines(old,(db_line*)r->data,localignorelist)==RETFAIL){
+      if(compare_dbline(old,(db_line*)r->data,localignorelist)!=0){
 	changednew=list_append(changednew,r->data);
 	changedold=list_append(changedold,(void*)old);
 	r->data=NULL;
@@ -744,10 +699,17 @@ void compare_db(list* new,db_config* conf)
 	nchg++;
       }else {
 	/* This line was ok */
-	free_db_line(old);
-	free(old);
-	free_db_line((db_line*)r->data);
-	free((db_line*)r->data);
+	/*
+	  Cannot free, chech why.
+	  It's because db_disk needs it for going back
+	  to it's parent.
+	  
+	  free_db_line(old);
+	  free(old);
+	  free_db_line((db_line*)r->data);
+	  free((db_line*)r->data);
+	*/
+	
 	l=list_delete_item(r);
       }
     }
@@ -767,15 +729,8 @@ void compare_db(list* new,db_config* conf)
 
 
   if(nadd!=0||nrem!=0||nchg!=0){
-    struct tm* st=localtime(&(conf->start_time));
-    
-    error(0,_("AIDE found differences between database and filesystem!!\n"));
-    error(5,_("Start timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
-	  st->tm_year+1900, st->tm_mon+1, st->tm_mday,
-	  st->tm_hour, st->tm_min, st->tm_sec);
-    error(0,_("Summary:\nTotal number of files=%i,added files=%i"
-	  ",removed files=%i,changed files=%i\n\n"),nfil,nadd,nrem,nchg);
-    
+    print_report_header(nfil,nadd,nrem,nchg);
+
     if(nadd!=0){
       error(5,_("Added files:\n"));
       for(r=added;r;r=r->next){
@@ -824,9 +779,101 @@ void compare_db(list* new,db_config* conf)
       }
     }
     conf->end_time=time(&(conf->end_time));
-    st=localtime(&(conf->end_time));
-    error(20,_("\nEnd timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
-	  st->tm_year+1900, st->tm_mon+1, st->tm_mday,
-	  st->tm_hour, st->tm_min, st->tm_sec);
+    print_report_footer(localtime(&(conf->end_time)));
   }
 }
+
+long report_tree(seltree* node,int stage, int* stat)
+{
+  list* r=NULL;
+  int ignorelist=0;
+  int top=0;
+
+  ignorelist=get_groupval("ignore_list");
+  
+  if (ignorelist==-1) {
+    ignorelist=0;
+  }
+  
+  if(stat[0]){
+    stat[0]=0;
+    top=1;
+  }
+
+
+  /* First check the tree for changes and do a bit of painting, 
+     then we print the terse report one changetype at a time
+     and then we do the detailed listing for changed nodes
+  */
+  if(stage==0){
+    /* If this node has been touched checked !=0 
+       If checked == 0 there is nothing to report
+    */
+    if(node->checked!=0){
+      stat[1]++;
+      if((node->checked&DB_OLD)&&(node->checked&DB_NEW)&&
+	 (node->old_data==NULL)&&(node->new_data==NULL)){
+	/* Node was added to twice and discovered to be not changed*/
+      }else if(!(node->checked&DB_OLD)&&(node->checked&DB_NEW)){
+	/* File is in new db but not old. (ADDED) */
+	stat[2]++;
+	node->checked|=NODE_ADDED;
+      }else if((node->checked&DB_OLD)&&!(node->checked&DB_NEW)){
+	/* File is in old db but not new. (REMOVED) */
+	stat[3]++;
+	node->checked|=NODE_REMOVED;
+      }else if((node->checked&DB_OLD)&&(node->checked&DB_NEW)&&
+	       (node->old_data!=NULL)&&(node->new_data!=NULL)){
+	/* File is in both db's and the data is still there. (CHANGED) */
+	stat[4]++;
+	node->checked|=NODE_CHANGED;
+      }
+    }
+  }
+
+  if((stage==1)&&stat[2]){
+    if(top)
+      error(5,_("Added files:\n"));
+    if(node->checked&NODE_ADDED){
+      error(5,_("added:%s\n"),node->new_data->filename);
+    }
+  }
+
+  if((stage==2)&&stat[3]){
+    if(top)
+      error(5,_("Removed files:\n"));
+    if(node->checked&NODE_REMOVED){
+      error(5,_("removed:%s\n"),node->old_data->filename);
+    }
+  }
+
+  if((stage==3)&&stat[4]){
+    if(top)
+      error(5,_("Changed files:\n"));
+    if(node->checked&NODE_CHANGED){
+      error(5,_("changed:%s\n"),node->new_data->filename);
+    }
+  }
+
+  if((stage==4)&&(conf->verbose_level>=5)&&stat[4]){
+    if(top)
+      error(5,_("Detailed information about changes:\n"));
+    if(node->checked&NODE_CHANGED){
+      print_dbline_changes(node->old_data,node->new_data,ignorelist);
+    }
+  }
+
+  /* All stage dependent things done for this node. Let's check children */
+  for(r=node->childs;r;r=r->next){
+    report_tree((seltree*)r->data,stage,stat);
+  }
+
+  if(top&&(stage==0)&&((stat[2]+stat[3]+stat[4])>0)){
+    print_report_header(stat[1],stat[2],stat[3],stat[4]);
+  }
+  
+  return (stat[2]+stat[3]+stat[4]);
+}
+
+const char* aide_key_9=CONFHMACKEY_09;
+const char* db_key_9=DBHMACKEY_09;
