@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/db.c,v 1.46 2001/11/26 07:13:42 pablo Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/src/db.c,v 1.9 2002/05/29 08:04:27 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
  
-
+#include "../config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include "db.h"
 #include "db_file.h"
+#include "db_disk.h"
 
 #ifdef WITH_PSQL
 #include "db_sql.h"
@@ -33,8 +34,9 @@
 #include "report.h"
 #include "be.h"
 
+/*
 #include <gcrypt.h>
-
+*/
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
@@ -46,19 +48,28 @@
 #include "locale-aide.h"
 /*for locale support*/
 
-db_line* db_char2line(char** ss,db_config* conf);
+db_line* db_char2line(char** ss,int db);
 long readint(char* s,char* err);
 long readoct(char* s,char* err);
 
 time_t base64totime_t(char*);
-int db_init(db_config* conf,int inout)
+int db_init(int db)
 {
   void* rv=NULL;
   
-  error(200,"db_init %i\n",inout);
+  error(200,"db_init %i\n",db);
   
-  if(inout==1){
-    
+  switch(db) {
+
+  case DB_DISK: {
+    /*
+      Should we actually do something here?
+     */
+    return db_disk_init();
+  }
+
+
+  case DB_OLD: {
     rv=be_init(1,conf->db_in_url,0);
     if(rv==NULL) {
       error(200,_("db_in is null\n"));      
@@ -68,8 +79,7 @@ int db_init(db_config* conf,int inout)
     error(200,_("db_in is nonnull\n"));
     return RETOK;
   }
-  if(inout==0){
-    
+  case DB_WRITE: {    
 #ifdef WITH_ZLIB
     if(conf->gzip_dbout){
        rv=be_init(0,conf->db_out_url,conf->gzip_dbout);
@@ -77,7 +87,7 @@ int db_init(db_config* conf,int inout)
     }
     else{
 #endif
-      rv=be_init(0,conf->db_out_url,conf->gzip_dbout);
+      rv=be_init(0,conf->db_out_url,0);
       conf->db_out=rv;
 #ifdef WITH_ZLIB
     }
@@ -90,29 +100,69 @@ int db_init(db_config* conf,int inout)
     error(200,_("db_out is nonnull %s\n"),conf->db_out_url->value);
     return RETOK;
   }
+  case DB_NEW: {
+    rv=be_init(1,conf->db_new_url,0);
+    if(rv==NULL) {
+      error(200,_("db_new is null\n"));      
+      return RETFAIL;
+    }
+    conf->db_new=rv;
+    error(200,_("db_new is nonnull\n"));
+    return RETOK;
+  }
+  }
   return RETFAIL;
 }
 
-db_line* db_readline(db_config* conf){
+db_line* db_readline(int db){
   db_line* s=NULL;
   int i=0;
+  url_t* db_url=NULL;
+  FILE** db_filep=NULL;
+  int* db_osize=0;
+  DB_FIELD** db_order=NULL;
 
-  switch (conf->db_in_url->type) {
+  switch (db) {
+  case DB_DISK: {
+    /*
+      Nothing else to be done?
+     */
+    s=db_readline_disk(db);
+    return s;
+  }
+  
+  case DB_OLD: {
+    db_url=conf->db_in_url;
+    db_filep=&(conf->db_in);
+    db_osize=&(conf->db_in_size);
+    db_order=&(conf->db_in_order);
+    break;
+  }
+  case DB_NEW: {
+    db_url=conf->db_new_url;
+    db_filep=&(conf->db_new);
+    db_osize=&(conf->db_new_size);
+    db_order=&(conf->db_new_order);
+    break;
+  }
+  }
+
+  switch (db_url->type) {
   case url_stdin:
   case url_fd:
   case url_file: {
     /* Should set errno */
     /* Please FIXME */
-    if (conf->db_in!=NULL) {
-      char** ss=db_readline_file(conf);
+    if ((*db_filep)!=NULL) {
+      char** ss=db_readline_file(db);
       if (ss!=NULL){
-	s=db_char2line(ss,conf);
+	s=db_char2line(ss,db);
 
-	for(i=0;i<conf->db_in_size;i++){
-	  if(conf->db_in_order[i]!=db_unknown && 
-	     ss[conf->db_in_order[i]]!=NULL){
-	    free(ss[conf->db_in_order[i]]);
-	    ss[conf->db_in_order[i]]=NULL;
+	for(i=0;i<*db_osize;i++){
+	  if((*db_order)[i]!=db_unknown && 
+	     ss[(*db_order)[i]]!=NULL){
+	    free(ss[(*db_order)[i]]);
+	    ss[(*db_order)[i]]=NULL;
 	  }
 	}
 	free(ss);
@@ -125,7 +175,7 @@ db_line* db_readline(db_config* conf){
 #ifdef WITH_PSQL
   case url_sql: {
     error(255,"db_sql readline...");
-    s=db_readline_sql(conf);
+    s=db_readline_sql(db);
     
     break;
   }
@@ -148,10 +198,32 @@ byte* base64tobyte(char* src,int len)
   return NULL;
 }
 
-db_line* db_char2line(char** ss,db_config* conf){
+db_line* db_char2line(char** ss,int db){
 
   int i;
   db_line* line=(db_line*)malloc(sizeof(db_line)*1);
+  url_t* db_url=NULL;
+  FILE** db_filep=NULL;
+  int* db_osize=0;
+  DB_FIELD** db_order=NULL;
+
+  switch (db) {
+  case DB_OLD: {
+    db_url=conf->db_in_url;
+    db_filep=&(conf->db_in);
+    db_osize=&(conf->db_in_size);
+    db_order=&(conf->db_in_order);
+    break;
+  }
+  case DB_NEW: {
+    db_url=conf->db_new_url;
+    db_filep=&(conf->db_new);
+    db_osize=&(conf->db_new_size);
+    db_order=&(conf->db_new_order);
+    break;
+  }
+  }
+
 
   line->md5=NULL;
   line->sha1=NULL;
@@ -181,12 +253,12 @@ db_line* db_char2line(char** ss,db_config* conf){
   
   line->attr=conf->attr; /* attributes from @@dbspec */
 
-  for(i=0;i<conf->db_in_size;i++){
-    switch (conf->db_in_order[i]) {
+  for(i=0;i<*db_osize;i++){
+    switch ((*db_order)[i]) {
     case db_filename : {
-      if(ss[conf->db_in_order[i]]!=NULL){
-	decode_string(ss[conf->db_in_order[i]]);
-	line->filename=strdup(ss[conf->db_in_order[i]]);
+      if(ss[(*db_order)[i]]!=NULL){
+	decode_string(ss[(*db_order)[i]]);
+	line->filename=strdup(ss[(*db_order)[i]]);
       } else {
 	error(0,"db_char2line():Error while reading database\n");
 	abort();
@@ -194,18 +266,19 @@ db_line* db_char2line(char** ss,db_config* conf){
       break;
     }
     case db_linkname : {
-      char *s = ss[conf->db_in_order[i]];
-      if(ss[conf->db_in_order[i]]!=NULL){
-	if(ss[conf->db_in_order[i]][0]=='0'){
-	  if(ss[conf->db_in_order[i]][1]=='\0'){
+      char *s = ss[(*db_order)[i]];
+      if(ss[(*db_order)[i]]!=NULL){
+	if(ss[(*db_order)[i]][0]=='0'){
+	  if(ss[(*db_order)[i]][1]=='\0'){
 	    line->linkname=NULL;
 	    break;
-	  }else if(ss[conf->db_in_order[i]][1]=='-'){
+	  }else if(ss[(*db_order)[i]][1]=='-'){
 	    line->linkname=strdup("");
 	    break;
-	  }else if(ss[conf->db_in_order[i]][1]=='0'){
-	    memmove(s,s+1,strlen(s+1)+1); // Hope this removes core
-	    // dumpping in some environments. Has something to do with
+	  }else if(ss[(*db_order)[i]][1]=='0'){
+	    memmove(s,s+1,strlen(s+1)+1); 
+	    // Hope this removes core
+	    // dumping in some environments. Has something to do with
 	    // memory (de)allocation.
 	  }
 	}
@@ -218,77 +291,77 @@ db_line* db_char2line(char** ss,db_config* conf){
       break;
     }
     case db_mtime : {
-      line->mtime=base64totime_t(ss[conf->db_in_order[i]]);
+      line->mtime=base64totime_t(ss[(*db_order)[i]]);
       break;
     }
     case db_bcount : {
-      line->bcount=readint(ss[conf->db_in_order[i]],"bcount");
+      line->bcount=readint(ss[(*db_order)[i]],"bcount");
       break;
     }
     case db_atime : {
-      line->atime=base64totime_t(ss[conf->db_in_order[i]]);
+      line->atime=base64totime_t(ss[(*db_order)[i]]);
       break;
     }
     case db_ctime : {
-      line->ctime=base64totime_t(ss[conf->db_in_order[i]]);
+      line->ctime=base64totime_t(ss[(*db_order)[i]]);
       break;
     }
     case db_inode : {
-      line->inode=readint(ss[conf->db_in_order[i]],"inode");
+      line->inode=readint(ss[(*db_order)[i]],"inode");
       break;
     }
 
     case db_uid : {
-      line->uid=readint(ss[conf->db_in_order[i]],"uid");
+      line->uid=readint(ss[(*db_order)[i]],"uid");
       break;
     }
     case db_gid : {
-      line->gid=readint(ss[conf->db_in_order[i]],"gid");
+      line->gid=readint(ss[(*db_order)[i]],"gid");
       break;
     }
     case db_size : {
-      line->size=readint(ss[conf->db_in_order[i]],"size");
+      line->size=readint(ss[(*db_order)[i]],"size");
       break;
     }
     case db_md5 : {
-      line->md5=base64tobyte(ss[conf->db_in_order[i]],
-			     strlen(ss[conf->db_in_order[i]]));
+      line->md5=base64tobyte(ss[(*db_order)[i]],
+			     strlen(ss[(*db_order)[i]]));
       break;
     }
     case db_sha1 : {
-      line->sha1=base64tobyte(ss[conf->db_in_order[i]],
-			      strlen(ss[conf->db_in_order[i]]));
+      line->sha1=base64tobyte(ss[(*db_order)[i]],
+			      strlen(ss[(*db_order)[i]]));
       break;
     }
     case db_rmd160 : {
-      line->rmd160=base64tobyte(ss[conf->db_in_order[i]],
-				strlen(ss[conf->db_in_order[i]]));
+      line->rmd160=base64tobyte(ss[(*db_order)[i]],
+				strlen(ss[(*db_order)[i]]));
       break;
     }
     case db_tiger : {
-      line->tiger=base64tobyte(ss[conf->db_in_order[i]],
-			       strlen(ss[conf->db_in_order[i]]));
+      line->tiger=base64tobyte(ss[(*db_order)[i]],
+			       strlen(ss[(*db_order)[i]]));
       break;
     }
 #ifdef WITH_MHASH
     case db_crc32 : {
-      line->crc32=base64tobyte(ss[conf->db_in_order[i]],
-			       strlen(ss[conf->db_in_order[i]]));
+      line->crc32=base64tobyte(ss[(*db_order)[i]],
+			       strlen(ss[(*db_order)[i]]));
       break;
     }
     case db_gost : {
-      line->gost=base64tobyte(ss[conf->db_in_order[i]],
-			       strlen(ss[conf->db_in_order[i]]));
+      line->gost=base64tobyte(ss[(*db_order)[i]],
+			       strlen(ss[(*db_order)[i]]));
       break;
     }
     case db_haval : {
-      line->haval=base64tobyte(ss[conf->db_in_order[i]],
-			       strlen(ss[conf->db_in_order[i]]));
+      line->haval=base64tobyte(ss[(*db_order)[i]],
+			       strlen(ss[(*db_order)[i]]));
       break;
     }
     case db_crc32b : {
-      line->crc32b=base64tobyte(ss[conf->db_in_order[i]],
-			       strlen(ss[conf->db_in_order[i]]));
+      line->crc32b=base64tobyte(ss[(*db_order)[i]],
+			       strlen(ss[(*db_order)[i]]));
       break;
     }
 #endif
@@ -298,8 +371,8 @@ db_line* db_char2line(char** ss,db_config* conf){
       int entries,lc;
       line->acl=NULL;
       
-      entries=strtol(ss[conf->db_in_order[i]],&endp,10);
-      if (endp==ss[conf->db_in_order[i]]) {
+      entries=strtol(ss[(*db_order)[i]],&endp,10);
+      if (endp==ss[(*db_order)[i]]) {
  	/* Something went wrong */
 	break;
       }
@@ -321,27 +394,27 @@ db_line* db_char2line(char** ss,db_config* conf){
     }
 #endif
     case db_perm : {
-      line->perm=readoct(ss[conf->db_in_order[i]],"permissions");
+      line->perm=readoct(ss[(*db_order)[i]],"permissions");
       break;
     }
     
     case db_lnkcount : {
-      line->nlink=readint(ss[conf->db_in_order[i]],"nlink");
+      line->nlink=readint(ss[(*db_order)[i]],"nlink");
       break;
     }
 
     case db_attr : {
-      line->attr=readint(ss[conf->db_in_order[i]],"attr");
+      line->attr=readint(ss[(*db_order)[i]],"attr");
       break;
     }
-
+    
     case db_unknown : {
       /* Unknown fields are ignored. */
       break;
     }
-  
+    
     default : {
-      error(0,_("Not implemented in db_char2line %i \n"),conf->db_in_order[i]);
+      error(0,_("Not implemented in db_char2line %i \n"),(*db_order)[i]);
       return NULL;
     }
     
@@ -516,3 +589,33 @@ int db_close(db_config* conf)
   }
   return RETFAIL;
 }
+
+void free_db_line(db_line* dl)
+{
+  if (dl==NULL) {
+    return;
+  }
+  
+#define checked_free(x) if(x!=NULL) { free(x); x=NULL; }
+
+  checked_free(dl->md5);
+  checked_free(dl->sha1);
+  checked_free(dl->rmd160);
+  checked_free(dl->tiger);
+  checked_free(dl->filename);
+  checked_free(dl->linkname);
+  
+  checked_free(dl->crc32);
+  checked_free(dl->crc32b);
+  checked_free(dl->gost);
+  checked_free(dl->haval);
+  /*
+    Possible bug.
+    Maybe we shouldn't free filename and linkname...
+   */
+  checked_free(dl->filename);
+  checked_free(dl->linkname);
+  
+}
+const char* aide_key_5=CONFHMACKEY_05;
+const char* db_key_5=DBHMACKEY_05;
