@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/db_file.c,v 1.59 2002/02/10 19:10:32 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/src/db_file.c,v 1.18 2002/05/30 09:42:46 pablo Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <time.h>
 
@@ -46,30 +47,25 @@
 #include <zlib.h>
 #endif
 
-#include <gcrypt.h>
-
 #define BUFSIZE 16384
 
-extern int db_scan(); /* Rumaa.... */
-extern char* dbtext; /* Todella rumaa... */
-extern long db_lineno;
-
-
-extern void db_buff(FILE*);
+#include "md.h"
 
 #ifdef WITH_ZLIB
 #define ZBUFSIZE 16384
 
-void handle_gzipped_input(int out){
+
+/* FIXME get rid of this */
+void handle_gzipped_input(int out,gzFile* gzp){
 
   int nread=0;
   int err=0;
   int* buf=malloc(ZBUFSIZE);
   buf[0]='\0';
   error(200,"handle_gzipped_input(),%d\n",out);
-  while(!gzeof(conf->db_gzin)){
-    if((nread=gzread(conf->db_gzin,buf,ZBUFSIZE))<0){
-      error(0,"gzread() failed!\n");
+  while(!gzeof(*gzp)){
+    if((nread=gzread(*gzp,buf,ZBUFSIZE))<0){
+      error(0,_("gzread() failed:gzerr=%s!\n"),gzerror(*gzp,&err));
       exit(1);
     } else {
       /* gzread returns 0 even if uncompressed bytes were read*/
@@ -80,7 +76,7 @@ void handle_gzipped_input(int out){
       }
       error(240,"nread=%d,strlen(buf)=%d,errno=%s,gzerr=%s\n",
 	    nread,strlen((char*)buf),strerror(errno),
-	    gzerror(conf->db_gzin,&err));
+	    gzerror(*gzp,&err));
       buf[0]='\0';
     }
   }
@@ -117,22 +113,38 @@ int dofprintf( const char* s,...)
   va_list ap;
   
   va_start(ap,s);
+  temp=(char*)malloc(3);
+  if(temp==NULL){
+    error(0,"Unable to alloc %i bytes\n",3);
+    return -1;
+  }
+
+  retval=vsnprintf(temp,3,s,ap);
+  
+  free(temp);
+  
+  temp=(char*)malloc(retval+2);
+  if(temp==NULL){
+    error(0,"Unable to alloc %i bytes\n",retval+2);
+    return -1;
+  }  
+  retval=vsnprintf(temp,retval+1,s,ap);
+  
+#ifdef WITH_MHASH
+  if(conf->do_dbnewmd)
+    mhash(conf->dbnewmd,(void*)temp,retval);
+#endif
+
 #ifdef WITH_ZLIB
   if(conf->gzip_dbout){
-    temp=(char*)malloc(BUFSIZE);
-    if(temp==NULL){
-      error(0,"Unable to alloc %i bytes\n",BUFSIZE);
-      return -1;
-    }
-    retval=vsnprintf(temp,BUFSIZE,s,ap);
     retval=gzwrite(conf->db_gzout,temp,retval);
-    free(temp);
   }else{
 #endif
     retval=vfprintf(conf->db_out,s,ap);
 #ifdef WITH_ZLIB
   }
 #endif
+  free(temp);
   va_end(ap);
 
   return retval;
@@ -140,11 +152,43 @@ int dofprintf( const char* s,...)
 
 
 
-int db_file_read_spec(db_config* conf){
+int db_file_read_spec(int db){
   
   int i=0;
+  int* db_osize=0;
+  DB_FIELD** db_order=NULL;
+  FILE** db_filep=NULL;
+  url_t* db_url=NULL;
+#ifdef WITH_ZLIB
+  gzFile* db_gzp=NULL;
+#endif
 
-  conf->db_in_order=(DB_FIELD*) malloc(1*sizeof(DB_FIELD));
+  switch (db) {
+  case DB_OLD: {
+    db_osize=&(conf->db_in_size);
+    db_order=&(conf->db_in_order);
+    db_filep=&(conf->db_in);
+    db_url=conf->db_in_url;
+    db_lineno=&db_in_lineno;
+#ifdef WITH_ZLIB
+    db_gzp=&(conf->db_gzin);
+#endif
+    break;
+  }
+  case DB_NEW: {
+    db_osize=&(conf->db_new_size);
+    db_order=&(conf->db_new_order);
+    db_filep=&(conf->db_new);
+    db_url=conf->db_new_url;
+    db_lineno=&db_new_lineno;
+#ifdef WITH_ZLIB
+    db_gzp=&(conf->db_gznew);
+#endif
+    break;
+  }
+  }
+
+  *db_order=(DB_FIELD*) malloc(1*sizeof(DB_FIELD));
   
   while ((i=db_scan())!=TNEWLINE){
     switch (i) {
@@ -155,25 +199,28 @@ int db_file_read_spec(db_config* conf){
 
       /* Yes... we do not check if realloc returns nonnull */
 
-      conf->db_in_order=(DB_FIELD*)
-	realloc((void*)conf->db_in_order,
-		(conf->db_in_size+1)*sizeof(DB_FIELD));
+      *db_order=(DB_FIELD*)
+	realloc((void*)*db_order,
+		((*db_osize)+1)*sizeof(DB_FIELD));
       
+      if(*db_order==NULL){
+	return RETFAIL;
+      }
       
-      conf->db_in_order[conf->db_in_size]=db_unknown;
+      (*db_order)[*db_osize]=db_unknown;
       
       for (l=0;l<db_unknown;l++){
 	
 	if (strcmp(db_names[l],dbtext)==0) {
 	  
-	  if (check_db_order(conf->db_in_order, conf->db_in_size,
+	  if (check_db_order(*db_order, *db_osize,
 			     db_value[l])==RETFAIL) {
 	    error(0,"Field %s redefined in @@dbspec\n",dbtext);
-	    conf->db_in_order[conf->db_in_size]=db_unknown;
+	    (*db_order)[*db_osize]=db_unknown;
 	  } else {
-	    conf->db_in_order[conf->db_in_size]=db_value[l];
+	    (*db_order)[*db_osize]=db_value[l];
 	  }
-	  conf->db_in_size++;
+	  (*db_osize)++;
 	  break;
 	}
       }
@@ -181,20 +228,20 @@ int db_file_read_spec(db_config* conf){
 	
 	if (strcmp(db_namealias[l],dbtext)==0) {
 	  
-	  if (check_db_order(conf->db_in_order, conf->db_in_size,
+	  if (check_db_order(*db_order, *db_osize,
 			     db_aliasvalue[l])==RETFAIL) {
 	    error(0,"Field %s redefined in @@dbspec\n",dbtext);
-	    conf->db_in_order[conf->db_in_size]=db_unknown;
+	    (*db_order)[*db_osize]=db_unknown;
 	  } else {
-	    conf->db_in_order[conf->db_in_size]=db_aliasvalue[l];
+	    (*db_order)[*db_osize]=db_aliasvalue[l];
 	  }
-	  conf->db_in_size++;
+	  (*db_osize)++;
 	  break;
 	}
       }
       if(l==db_unknown){
 	error(0,"Unknown field %s in database\n",dbtext);
-	conf->db_in_size++;
+	(*db_osize)++;
       }
       break;
     }
@@ -212,11 +259,11 @@ int db_file_read_spec(db_config* conf){
     }
   }
 
-  /* Lets generate attr from db_in_order if database does not have attr */
+  /* Lets generate attr from db_order if database does not have attr */
   conf->attr=-1;
 
-  for (i=0;i<conf->db_in_size;i++) {
-    if (conf->db_in_order[i]==db_attr) {
+  for (i=0;i<*db_osize;i++) {
+    if ((*db_order)[i]==db_attr) {
       conf->attr=1;
     }
   }
@@ -224,13 +271,13 @@ int db_file_read_spec(db_config* conf){
     conf->attr=0;
     error(0,"Database does not have attr field.\nComparation may be incorrect\nGenerating attr-field from dbspec\nIt might be a good Idea to regenerate databases. Sorry.\n");
     for(i=0;i<conf->db_in_size;i++) {
-      conf->attr|=1<<conf->db_in_order[i];
+      conf->attr|=1<<(*db_order)[i];
     }
   }
   return RETOK;
 }
 
-char** db_readline_file(db_config* conf){
+char** db_readline_file(int db){
   
   char** s=NULL;
   
@@ -238,9 +285,52 @@ char** db_readline_file(db_config* conf){
   int r;
   int a=0;
   int token=0;
+  int gotbegin_db=0;
+  int gotend_db=0;
+  int* domd=NULL;
+  MHASH* md=NULL;
+  char** oldmdstr=NULL;
+  int* db_osize=0;
+  DB_FIELD** db_order=NULL;
+  FILE** db_filep=NULL;
+  url_t* db_url=NULL;
+#ifdef WITH_ZLIB
+  gzFile* db_gzp=NULL;
+#endif
+
+  switch (db) {
+  case DB_OLD: {
+    md=&(conf->dboldmd);
+    domd=&(conf->do_dboldmd);
+    oldmdstr=&(conf->old_dboldmdstr);
+    db_osize=&(conf->db_in_size);
+    db_order=&(conf->db_in_order);
+    db_filep=&(conf->db_in);
+    db_url=conf->db_in_url;
+    db_lineno=&db_in_lineno;
+#ifdef WITH_ZLIB
+    db_gzp=&(conf->db_gzin);
+#endif
+    break;
+  }
+  case DB_NEW: {
+    md=&(conf->dbnewmd);
+    domd=&(conf->do_dbnewmd);
+    oldmdstr=&(conf->old_dbnewmdstr);
+    db_osize=&(conf->db_new_size);
+    db_order=&(conf->db_new_order);
+    db_filep=&(conf->db_new);
+    db_url=conf->db_new_url;
+    db_lineno=&db_new_lineno;
+#ifdef WITH_ZLIB
+    db_gzp=&(conf->db_gznew);
+#endif
+    break;
+  }
+  }
   
-  if (conf->db_in_size==0) {
-    db_buff(conf->db_in);
+  if (*db_osize==0) {
+    db_buff(db,*db_filep);
     
     token=db_scan();
     while((token!=TDBSPEC)){
@@ -250,36 +340,25 @@ char** db_readline_file(db_config* conf){
 	continue;
 	break;
       }
+      case TBEGIN_DB: {
+	token=db_scan();
+	gotbegin_db=1;
+	continue;
+	break;
+      }
+      case TNEWLINE: {
+	if(gotbegin_db){
+	  *domd=1;
+	  token=db_scan();
+	  continue;
+	}else {
+	  token=TEOF;
+	  break;
+	}
+      }
       case TGZIPHEADER: {
-
-#ifndef WITH_ZLIB
-	error(0,"Zlib support is not compiled in.\n");
+	error(0,"Gzipheader found inside uncompressed db!\n");
 	return NULL;
-#else
-	int pipefd[2];
-	pid_t retval=0;
-	/* Is this safe? Can we always fseek back? Probably.*/
-	fseek(conf->db_in,0L,SEEK_SET);
-	if(pipe(pipefd)<0){
-	  error(0,"Couldn't open a pipe for zipped input\n");
-	  return NULL;
-	}
-	if((retval=fork())==0){
-	  /* The child process */
-	  close(pipefd[0]);
-	  conf->db_gzin=gzdopen(fileno(conf->db_in),"rb");
-	  /* WARNING This causes weird problems. Don't do it.
-	     fclose(conf->db_in);
-	  */
-	  handle_gzipped_input(pipefd[1]);
-	} else {
-	  /* The parent */
-	  fclose(conf->db_in);
-	  conf->db_in=fdopen(pipefd[0],"r");
-	  db_buff(conf->db_in);
-	  close(pipefd[1]);
-	}
-#endif
 	break;
       }
       default: {
@@ -296,12 +375,17 @@ char** db_readline_file(db_config* conf){
       token=db_scan();
     }
 
+    if(FORCEDBMD&&!gotbegin_db){
+      error(0,"Database %i does not have checksum!\n");
+      return NULL;
+    }
+
     if (token!=TDBSPEC) {
       /*
        * error.. must be a @@dbspec line
        */
       
-      switch (conf->db_in_url->type) {
+      switch (db_url->type) {
       case url_file : {
 	error(0,"Filedatabase must have one db_spec spesification\n");
 	break;
@@ -331,13 +415,16 @@ char** db_readline_file(db_config* conf){
      * Here we read da spec
      */
     
-    if (db_file_read_spec(conf)!=0) {
+    if (db_file_read_spec(db)!=0) {
       /* somethin went wrong */
       return s;
     }
     
+  }else {
+    /* We need to switch the buffer cleanly*/
+    db_buff(db,NULL);
   }
-  
+
   s=(char**)malloc(sizeof(char*)*db_unknown);
 
   /* We NEED this to avoid Bus errors on Suns */
@@ -345,31 +432,24 @@ char** db_readline_file(db_config* conf){
     s[i]=NULL;
   }
   
-  for(i=0;i<conf->db_in_size;i++){
+  for(i=0;i<*db_osize;i++){
     switch (r=db_scan()) {
       
     case TDBSPEC : {
       
-      error(0,"Databasefile can have only one db_spec.\nTrying to continue on line %i\n",db_lineno);
-      
-      /*
-	if (conf->db_in_order[i]!=db_unknown) {
-	s[conf->db_in_order[i]]=(char*)strdup(dbtext);
-      }
-      */
-      
+      error(0,"Databasefile can have only one db_spec.\nTrying to continue on line %i\n",*db_lineno);      
       break;
     }
     case TNAME : {
-      if (conf->db_in_order[i]!=db_unknown) {
-	s[conf->db_in_order[i]]=(char*)strdup(dbtext);
+      if ((*db_order)[i]!=db_unknown) {
+	s[*db_order[i]]=(char*)strdup(dbtext);
       }
       break;
     }
     
     case TID : {
-      if (conf->db_in_order[i]!=db_unknown) {
-	s[conf->db_in_order[i]]=(char*)strdup(dbtext);
+      if ((*db_order)[i]!=db_unknown) {
+	s[(*db_order)[i]]=(char*)strdup(dbtext);
       }
       break;
     }
@@ -384,31 +464,74 @@ char** db_readline_file(db_config* conf){
       /*  */
 
       error(0,"Not enough parameters in db:%i. Trying to continue.\n",
-	    db_lineno);
+	    *db_lineno);
       for(a=0;a<i;a++){
-	free(s[conf->db_in_order[a]]);
-	s[conf->db_in_order[a]]=NULL;
+	free(s[(*db_order)[a]]);
+	s[(*db_order)[a]]=NULL;
       }
       i=0;
       break;
 
     }
 
+    case TBEGIN_DB : {
+      error(0,_("Corrupt db. Found @@begin_db inside db. Please check\n"));
+      return NULL;
+      break;
+    }
+
+    case TEND_DB : {
+      gotend_db=1;
+      token=db_scan();
+      if(token!=TSTRING){
+	error(0,_("Corrupt db. Checksum garbled\n"));
+	abort();
+      } else {
+	if(*md){
+	  byte* dig=NULL;
+	  char* digstr=NULL;
+	  
+	  *oldmdstr=strdup(dbtext);
+	  
+	  mhash(*md,NULL,0);
+	  dig=(byte*)
+	    malloc(sizeof(byte)*mhash_get_block_size(conf->dbhmactype));
+	  mhash_deinit(*md,(void*)dig);
+	  digstr=encode_base64(dig,mhash_get_block_size(conf->dbhmactype));
+	  if(strncmp(digstr,*oldmdstr,strlen(digstr))!=0){
+	    error(0,_("Db checksum mismatch for db:%i\n"),db);
+	    abort();
+	  }
+	}else {
+	  error(0,"@@end_db found without @@begin_db in db:%i\n",db);
+	  abort();
+	}
+      }
+      token=db_scan();
+      if(token!=TNEWLINE){
+	error(0,_("Corrupt db. Checksum garbled\n"));
+	abort();
+      }	
+      break;
+    }
+
     case TEOF : {
-	
+      if(gotend_db){
+	return NULL;
+      }	
       /* This can be the first token on a line */
       if(i>0){
-	error(0,"Not enough parameters in db:%i\n",db_lineno);
+	error(0,"Not enough parameters in db:%i\n",*db_lineno);
       };
       for(a=0;a<i;a++){
-	free(s[conf->db_in_order[a]]);
+	free(s[(*db_order)[a]]);
       }
       free(s);
       return NULL;
       break;
     }
     case TERROR : {
-      error(0,"There was an error in the database file on line:%i.\n",db_lineno);
+      error(0,"There was an error in the database file on line:%i.\n",*db_lineno);
       break;
     }
     
@@ -418,7 +541,7 @@ char** db_readline_file(db_config* conf){
       
       free(s);
       s=NULL;
-      i=conf->db_in_size;
+      i=*db_osize;
       break;
     }
     }
@@ -432,7 +555,7 @@ char** db_readline_file(db_config* conf){
   a=db_scan();
 
   if (a!=TNEWLINE&&a!=TEOF) {
-    error(0,"Newline exptexted in database. Readin until end of line\n");
+    error(0,"Newline expected in database. Readin until end of line\n");
     do {
       
       error(0,"Skipped value %s\n",dbtext);
@@ -473,11 +596,16 @@ int db_writechar(char* s,FILE* file,int i)
     }
     s++;
   }
-
-  r=CLEANDUP(s);
+  
+  if (!i && s[0]=='#') {
+    dofprintf("# ");
+    r=CLEANDUP(s+1);
+  } else {
+    r=CLEANDUP(s);
+  }
+  
   retval=dofprintf("%s",r);
   free(r);
-
   return retval;
 }
 
@@ -491,12 +619,18 @@ int db_writeint(long i,FILE* file,int a)
   
 }
 
-int db_write_byte_base64(byte*data,size_t len,FILE* file,int i )
+int db_write_byte_base64(byte*data,size_t len,FILE* file,int i,int th,
+			 int attr )
 {
   char* tmpstr=NULL;
   int retval=0;
   
-  tmpstr=encode_base64(data,len);
+  
+  if (data!=NULL&&th&attr) {
+    tmpstr=encode_base64(data,len);
+  } else {
+    tmpstr=NULL;
+  }
   if(i){
     dofprintf(" ");
   }
@@ -562,10 +696,36 @@ int db_writespec_file(db_config* conf)
   int i=0;
   int j=0;
   int retval=1;
+  void*key=NULL;
+  int keylen=0;
   struct tm* st;
   time_t tim=time(&tim);
   st=localtime(&tim);
 
+  retval=dofprintf("@@begin_db\n");
+  if(retval==0){
+    return RETFAIL;
+  }
+
+#ifdef WITH_MHASH
+  /* From hereon everything must MD'd before write to db */
+  if((key=get_db_key())!=NULL){
+    keylen=get_db_key_len();
+    conf->do_dbnewmd=1;
+    if( (conf->dbnewmd=
+	 mhash_hmac_init(conf->dbhmactype,
+			 key,
+			 keylen,
+			 mhash_get_hash_pblock(conf->dbhmactype)))==
+	MHASH_FAILED){
+      error(0, "mhash_hmac_init() failed for db write. Aborting\n");
+      abort();
+    }
+  }
+  
+  
+#endif
+
   retval=dofprintf(
 		 "# This file was generated by Aide, version %s\n"
 		 "# Time of generation was %.4u-%.2u-%.2u %.2u:%.2u:%.2u\n",
@@ -576,7 +736,14 @@ int db_writespec_file(db_config* conf)
   if(retval==0){
     return RETFAIL;
   }
-
+  if(conf->config_version){
+    retval=dofprintf(
+		     "# The config version used to generate this file was:\n"
+		     "# %s\n", conf->config_version);
+    if(retval==0){
+      return RETFAIL;
+    }
+  }
   retval=dofprintf("@@db_spec ");
   if(retval==0){
     return RETFAIL;
@@ -674,56 +841,66 @@ int db_writeline_file(db_line* line,db_config* conf){
     }
     case db_md5 : {
       db_write_byte_base64(line->md5,
-			   gcry_md_get_algo_dlen(GCRY_MD_MD5),conf->db_out,i);
+			   HASH_MD5_LEN,
+			   conf->db_out,i,
+			   DB_MD5,line->attr);
+	
       break;
     }
     case db_sha1 : {
       db_write_byte_base64(line->sha1,
-			   gcry_md_get_algo_dlen(GCRY_MD_SHA1),conf->db_out,i);
+			   HASH_SHA1_LEN,
+			   conf->db_out,i,
+			   DB_SHA1,line->attr);
+
       break;
     }
     case db_rmd160 : {
       db_write_byte_base64(line->rmd160,
-			   gcry_md_get_algo_dlen(GCRY_MD_RMD160),
-			   conf->db_out,i);
+			   HASH_RMD160_LEN,
+			   conf->db_out,i,
+			   DB_RMD160,line->attr);
       break;
     }
     case db_tiger : {
       db_write_byte_base64(line->tiger,
-			   gcry_md_get_algo_dlen(GCRY_MD_TIGER),
-			   conf->db_out,i);
+			   HASH_TIGER_LEN,
+			   conf->db_out,i,
+			   DB_TIGER,line->attr);
       break;
     }
     case db_perm : {
       db_writeoct(line->perm,conf->db_out,i);
       break;
     }
-#ifdef WITH_MHASH
     case db_crc32 : {
       db_write_byte_base64(line->crc32,
-			   mhash_get_block_size(MHASH_CRC32),
-			   conf->db_out,i);
+			   HASH_CRC32_LEN,
+			   conf->db_out,i,
+			   DB_CRC32,line->attr);
       break;
     }
     case db_crc32b : {
       db_write_byte_base64(line->crc32b,
-			   mhash_get_block_size(MHASH_CRC32B),
-			   conf->db_out,i);
+			   HASH_CRC32B_LEN,
+			   conf->db_out,i,
+			   DB_CRC32B,line->attr);
       break;
     }
     case db_haval : {
       db_write_byte_base64(line->haval,
-			   mhash_get_block_size(MHASH_HAVAL256),
-			   conf->db_out,i);
+			   HASH_HAVAL256_LEN,
+			   conf->db_out,i,
+			   DB_HAVAL,line->attr);
       break;
     }
     case db_gost : {
       db_write_byte_base64(line->gost ,
-			   mhash_get_block_size(MHASH_GOST),
-			   conf->db_out,i);
+			   HASH_GOST_LEN,
+			   conf->db_out,i,
+			   DB_GOST,line->attr);
       break;
     }
-#endif
     case db_attr : {
       db_writeint(line->attr,
 		  conf->db_out,i);
@@ -757,6 +934,32 @@ int db_writeline_file(db_line* line,db_config* conf){
 }
 
 int db_close_file(db_config* conf){
+  
+#ifdef WITH_MHASH
+  byte* dig=NULL;
+  char* digstr=NULL;
+
+  if(conf->db_out
+#ifdef WITH_ZLIB
+     || conf->db_gzout
+#endif
+     ){
+
+    /* Let's write @@end_db <checksum> */
+    if (conf->dbnewmd!=NULL) {
+      mhash(conf->dbnewmd, NULL ,0);
+      dig=(byte*)malloc(sizeof(byte)*mhash_get_block_size(conf->dbhmactype));
+      mhash_deinit(conf->dbnewmd,(void*)dig);
+      digstr=encode_base64(dig,mhash_get_block_size(conf->dbhmactype));
+      conf->do_dbnewmd=0;
+      dofprintf("@@end_db %s\n",digstr);
+      free(dig);
+      free(digstr);
+    } else {
+      dofprintf("@@end_db\n");
+    }
+  }
+#endif
 
 #ifndef WITH_ZLIB
   if(fclose(conf->db_out)){
@@ -779,3 +982,6 @@ int db_close_file(db_config* conf){
 
   return RETOK;
 }
+/*
+const char* aide_key_11=CONFHMACKEY_11;
+*/
