@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/list.c,v 1.14 2000/05/29 08:38:59 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/src/list.c,v 1.5 2002/05/29 08:04:27 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -126,6 +126,7 @@ list* new_list_item(void* data)
 /*
  * delete_list_item()
  * delete a item from list
+ * returns head of a list.
  */
 
 list* list_delete_item(list* item){
@@ -159,6 +160,7 @@ list* list_delete_item(list* item){
     r=item->prev;
     item->header->tail=r;
     r->next=NULL;
+    r=r->header->head;
     free(item);
     return r;
   }
@@ -171,6 +173,8 @@ list* list_delete_item(list* item){
     r=item->next;
     item->header->head=r;
     r->prev=NULL;
+    r=r->header->head;
+    
     free(item);
     return r;
   }
@@ -181,7 +185,11 @@ list* list_delete_item(list* item){
   item->next->prev=item->prev;
   
   free(item);
-
+  r=r->header->head;
+  
   return r;
   
 }
+/*
+const char* aide_key_12=KEY_12;
+*/
