@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/symboltable.c,v 1.5 2000/05/29 08:38:59 rammer Exp $
+ * Copyright (C) 1999,2000,2001,2002 Rami Lehti,Pablo Virolainen
+ * $Header: /cvs-root-aide/aide2/src/symboltable.c,v 1.5 2002/05/29 08:04:27 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
+#include "../config.h"
 #include <stdio.h>
+#include <string.h>
 #include "symboltable.h"
 /*for locale support*/
 #include "locale-aide.h"
@@ -47,3 +48,6 @@ list* list_find(char* s,list* item){
   }
   return NULL;
 }
+/*
+const char* aide_key_10=KEY_10;
+*/
